# coding=utf-8
import csv

import django_excel as excel
from django.http import HttpResponse

from notifications import get_today_midnight_date
from . import e500_error, wrap_response
from ..models import Subject, StudySubject, Appointment


def export_to_csv(request, data_type="subjects"):
    # Create the HttpResponse object with the appropriate CSV header.
    selected_fields = request.GET.get('fields', None)
    response = HttpResponse(content_type='text/csv')
    response['Content-Disposition'] = 'attachment; filename="' + data_type + '-' + get_today_midnight_date().strftime(
        "%Y-%m-%d") + '.csv"'

    if data_type == "subjects":
        data = get_subjects_as_array(selected_fields=selected_fields)
    elif data_type == "appointments":
        data = get_appointments_as_array(selected_fields=selected_fields)
    else:
        return e500_error(request)
    writer = csv.writer(response, quotechar=str(u'"'), quoting=csv.QUOTE_ALL)
    for row in data:
        writer.writerow([s.encode("utf-8") for s in row])

    return response


def export_to_excel(request, data_type="subjects"):
    selected_fields = request.GET.get('fields', None)
    filename = data_type + '-' + get_today_midnight_date().strftime("%Y-%m-%d") + ".xls"
    if data_type == "subjects":
        data = get_subjects_as_array(selected_fields=selected_fields)
    elif data_type == "appointments":
        data = get_appointments_as_array(selected_fields=selected_fields)
    else:
        return e500_error(request)

    response = excel.make_response_from_array(data, 'xls', file_name=filename)
    response['Content-Disposition'] = 'attachment; filename="' + filename + '"'

    return response


class CustomField:
    def __init__(self, dictionary):
        for k, v in dictionary.items():
            setattr(self, k, v)


DROP_OUT_FIELD = CustomField({'verbose_name': "DROP OUT", 'name': "custom-drop-out"})
APPOINTMENT_TYPE_FIELD = CustomField({
        'name': 'appointment_types',
        'verbose_name': 'Appointment Types'
    })
STUDY_SUBJECT_FIELDS = [CustomField({
        'name': 'nd_number',
        'verbose_name' : 'ND number'
    })]

SUBJECT_FIELDS = [CustomField({
        'name': 'last_name',
        'verbose_name': 'Family name'
    }),
    CustomField({
        'name': 'first_name',
        'verbose_name': 'Name'
    })]
VISIT_FIELDS = [CustomField({
        'name': 'visit_number',
        'verbose_name': 'Visit'
    })]

def filter_fields_from_selected_fields(fields, selected_fields):
    if selected_fields is None:
        return fields
    selected_fields = set(selected_fields.split(','))
    return [field for field in fields if field.name in selected_fields]

def get_default_subject_fields():
    subject_fields = []
    for field in Subject._meta.fields:
        if field.name.upper() != "ID":
            subject_fields.append(field)
    for field in StudySubject._meta.fields:
        if field.name.upper() != "ID" and field.name.upper() != "SUBJECT":
            subject_fields.append(field)
    subject_fields.append(DROP_OUT_FIELD)
    return subject_fields

def get_subjects_as_array(selected_fields=None):
    result = []
    subject_fields = get_default_subject_fields() 
    subject_fields = filter_fields_from_selected_fields(subject_fields, selected_fields)

    field_names = [field.verbose_name for field in subject_fields] #faster than loop
    result.append(field_names)

    subjects = StudySubject.objects.order_by('-subject__last_name')
    for subject in subjects:
        row = subject_to_row_for_fields(subject, subject_fields)
        result.append([unicode(s).replace("\n", ";").replace("\r", ";") for s in row])
    return result

def subject_to_row_for_fields(study_subject, subject_fields):
    row = []
    for field in subject_fields:
        if field == DROP_OUT_FIELD:
            if not study_subject.resigned:
                cell = False
            else:
                finished_appointments = Appointment.objects.filter(visit__subject=study_subject).filter(
                    status=Appointment.APPOINTMENT_STATUS_FINISHED).count()
                if finished_appointments > 0:
                    cell = True
                else:
                    cell = False
        else:
            if hasattr(study_subject, field.name):
                cell = getattr(study_subject, field.name)
            elif hasattr(study_subject.subject, field.name):
                cell = getattr(study_subject.subject, field.name)
            if cell is None:
                cell = ""
        row.append(cell)
    return row

def get_appointment_fields():
    appointments_fields = []
    for field in Appointment._meta.fields:
        if field.name.upper() != "VISIT" and field.name.upper() != "ID" and \
                        field.name.upper() != "WORKER_ASSIGNED" and field.name.upper() != "APPOINTMENT_TYPES" and \
                        field.name.upper() != "ROOM" and field.name.upper() != "FLYING_TEAM":
            appointments_fields.append(field)

    all_fields = STUDY_SUBJECT_FIELDS + SUBJECT_FIELDS + VISIT_FIELDS + appointments_fields + [APPOINTMENT_TYPE_FIELD]

    return all_fields, appointments_fields

def get_appointments_as_array(selected_fields=None):
    result = []
    all_fields, appointments_fields = get_appointment_fields()
    all_fields = filter_fields_from_selected_fields(all_fields, selected_fields)
    appointments_fields = filter_fields_from_selected_fields(appointments_fields, selected_fields)

    field_names = [field.verbose_name for field in all_fields] #faster than loop
    result.append(field_names)

    appointments = Appointment.objects.order_by('-datetime_when')

    for appointment in appointments:
        #add field_names ['ND number', 'Family name', 'Name', 'Visit'] first
        row = []
        for field in STUDY_SUBJECT_FIELDS:
            if field.verbose_name in field_names:
                if appointment.visit is None:
                    row.append('---')
                else:
                    row.append(getattr(appointment.visit.subject, field.name))
        for field in SUBJECT_FIELDS:
            if field.verbose_name in field_names:
                if appointment.visit is None:
                    row.append('---')
                else:
                    row.append(getattr(appointment.visit.subject.subject, field.name))
        for field in VISIT_FIELDS:
            if field.verbose_name in field_names:
                if appointment.visit is None:
                    row.append('---')
                else:
                    row.append(getattr(appointment.visit, field.name))
        for field in appointments_fields:
            row.append(getattr(appointment, field.name))
        if APPOINTMENT_TYPE_FIELD.verbose_name in field_names:
            #avoid last comma in the list of appointment types
            type_string = ','.join([appointment_type.code for appointment_type in appointment.appointment_types.all()])
            row.append(type_string)
            result.append([unicode(s).replace("\n", ";").replace("\r", ";") for s in row])
    return result


def export(request):
    return wrap_response(request, 'export/index.html', {
        'subject_fields': get_default_subject_fields(),
        'appointment_fields': get_appointment_fields()[0]
    })